# encoding: utf-8

# ============================[ General settings ]============================
SUPPORTED_FORMATS = ['png', 'jpg', 'jpeg']
DEFAULT_QUALITY = 80
DEFAULT_BG_COLOR = (255, 255, 255)
MIN_BIG_IMG_SIZE = 80_000
MIN_BIG_IMG_AREA = 800 * 600

# ====================[ iOS/Pythonista specific settings ]====================
IPAD_FONT_SIZE = 15
IPHONE_FONT_SIZE = 10
IOS_WORKERS = 2
IOS_FONT = "Menlo"

